﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Forms;

using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace RestrictedExtentSample
{
	public partial class MapForm : Form
	{
		private readonly InMemoryFeatureLayer _featureLayer = new InMemoryFeatureLayer();
		private readonly GdiPlusRasterLayer _backgroundImageLayer = new GdiPlusRasterLayer();

		public MapForm()
		{
			this.InitializeComponent();
			this.InitializeMapControl();
		}

		private void InitializeMapControl()
		{
			this.map.MapResizeMode = MapResizeMode.PreserveExtent;
			this.map.MapUnit = GeographyUnit.Meter;
			this.map.MinimumScale = 0;
			this.map.AdornmentOverlay.ShowLogo = false;

			this.map.ZoomLevelSet = MapUtility.PartitionZoomLevelSet(5, this.map.ZoomLevelSet);

			this.AddMapBackgroundImage();
			this.AddFeatureLayer();
		}

		private void AddMapBackgroundImage()
		{
			this._backgroundImageLayer.PathFilename = "MapSampleBackground.png";

			LayerOverlay overlay = new LayerOverlay();
			overlay.Layers.Add(this._backgroundImageLayer);

			this.map.Overlays.Add(overlay);
		}

		private void AddFeatureLayer()
		{
			this.RepopulateFeatureLayer();

			this._featureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
			this._featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

			LayerOverlay overlay = new LayerOverlay();
			overlay.Layers.Add(this._featureLayer);
			this.map.Overlays.Add(overlay);
		}

		private void RepopulateFeatureLayer()
		{
			this._backgroundImageLayer.Open();
			RectangleShape imageBox = this._backgroundImageLayer.GetBoundingBox();
			imageBox = imageBox.Buffer(-5, GeographyUnit.Meter, DistanceUnit.Meter).GetBoundingBox();

			this._featureLayer.Open();
			this._featureLayer.InternalFeatures.Clear();

			foreach (Feature feature in MapUtility.GetFeatures(10, imageBox))
			{
				this._featureLayer.InternalFeatures.Add(feature);
			}

			this.map.RestrictedExtent = this.CalculateMapBoundingRectangle();
		}

		private void ScaleMapToFit()
		{
			this.map.MaximumScale = double.MaxValue;
			this.map.CurrentExtent = this.map.RestrictedExtent;
			this.map.MaximumScale = this.map.CurrentScale;

			this.map.Refresh();
		}

		private RectangleShape CalculateMapBoundingRectangle()
		{
			this._featureLayer.Open();
			RectangleShape result = this._featureLayer.GetBoundingBox();
			result.ScaleUp(10);

			return result;
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			this.ScaleMapToFit();
		}

		private void scaleToFitButton_Click(object sender, EventArgs e)
		{
			this.ScaleMapToFit();
		}

		private void repopulateButton_Click(object sender, EventArgs e)
		{
			this.RepopulateFeatureLayer();
			this.ScaleMapToFit();
		}
	}
}