﻿using System;
using System.Linq;
using System.IO;
using ThinkGeo.MapSuite.Core;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace RestrictedExtentSample
{
	internal static class MapUtility
	{
		internal static IList<Feature> GetFeatures(int count, RectangleShape boundingBox)
		{
			Random randomizer = new Random();

			IList<Feature> result = new List<Feature>(count);
			for (int i = 0; i < count; i++)
			{
				int x = randomizer.Next((int)boundingBox.LowerLeftPoint.X, (int)boundingBox.LowerRightPoint.X);
				int y = randomizer.Next((int)boundingBox.LowerLeftPoint.Y, (int)boundingBox.UpperLeftPoint.Y);

				result.Add(new Feature(new Vertex(x, y)));
			}

			return result;
		}

		internal static ZoomLevelSet PartitionZoomLevelSet(int stepsBetweenZoomLevels, ZoomLevelSet referenceZoomLevelSet)
		{
			ZoomLevelSet partitionedZoomLevelSet = new ZoomLevelSet();
			Collection<ZoomLevel> zoomLevels = referenceZoomLevelSet.GetZoomLevels();

			partitionedZoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(zoomLevels[0].Scale));
			foreach (ZoomLevel zoomLevel in zoomLevels)
			{
				double lowerScale = ZoomLevelSet.GetLowerZoomLevelScale(zoomLevel.Scale, referenceZoomLevelSet);
				if (lowerScale != zoomLevel.Scale)
				{
					for (int i = 1; i < stepsBetweenZoomLevels + 2; i++)
					{
						double steppedScale = zoomLevel.Scale - (((zoomLevel.Scale - lowerScale) / (stepsBetweenZoomLevels + 1)) * i);
						partitionedZoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(steppedScale));
					}
				}
			}

			return partitionedZoomLevelSet;
		}
	}
}